<?php

namespace de\IPSymcon\ernie;

/**********************************************
 ***
 *** INTERFACE Debugger
 ***
 ***/
interface Debugger {
	public function showDebugMessage ($message);
}

/**********************************************
 ***
 *** IMPLEMENTATIONS OF INTERFACE Debugger
 ***
 ***/
class DebuggerVoid implements Debugger {
	public function showDebugMessage ($message) {
		// DO NOTHING
	}
}
class DebuggerEchoLinebreak implements Debugger {
	public function showDebugMessage ($message) {
		echo $message . "\n";
	}
}

class DebuggerEchoHTML implements Debugger {
	public function showDebugMessage ($message) {
		echo $message . "<br>\n";
	}
}

class DebuggerFile implements Debugger {
	public function showDebugMessage ($message) {
		echo "TODO: WRITE THIS TO AN FILE: ". $message . "\n";
	}
}

/**********************************************
 ***
 *** INTERFACE EnvironmentDebugger
 ***
 ***/
interface EnvironmentDebugger extends Debugger {
	public function showEnvironmentDebugMessage ($message, $debugCategorie, $debugCount);
	// public function showMessagesOnEnd() bzw. destructer;
	// public function __toString();
}

/******************************************************
 ***
 *** IMPLEMENTATIONS OF INTERFACE EnvironmentDebugger
 ***
 ***/
class DebugEnvironmentObject {
	public $environmentName;
	public $debugCount;
	public $showDebugWithPRE;
	public $showDebugCategories;
	public $showDebugCount;
	public $showDebugMessagesOnEnd;
	public $usedDebuggers;

	public function __construct($varEnvironmentName,$varDebugCount,$varShowDebugWithPRE,$varShowDebugCategories,$varShowDebugCount,$varShowDebugMessagesOnEnd,$varUsedDebuggers) {
		$this->environmentName 			= $varEnvironmentName;
		$this->debugCount 				= $varDebugCount;
		$this->showDebugWithPRE 		= $varShowDebugWithPRE;
		$this->showDebugCategories 	= $varShowDebugCategories;
		$this->showDebugCount			= $varShowDebugCount;
		$this->showDebugMessagesOnEnd = $varShowDebugMessagesOnEnd;
		$this->usedDebuggers 			= $varUsedDebuggers;
	}
}

define ('DEBUG_MODE_VOID'			,3);
define ('DEBUG_MODE_ECHO_LINEBREAK'	,5);
define ('DEBUG_MODE_ECHO_HTML'		,7);
define ('DEBUG_MODE_FILE'			,11);
// SEE __toString Method in class EnvironmentDebugObject !!

class EnvironmentDebugObject implements EnvironmentDebugger {
	/*
	 * Diese Klasse erweitert die vorherige Debugger Klasse um die funktionalitaet,
	 * dass beliebige DEBUG_ENVIRONMENTS festgelegt werden koennen.
	 * Somit kann jede Funktion bzw. Klasse verschiedene DEBUG_LEVEL besitzen sowie
	 * einzeln deklarierbare Ziele, wohin die Debug Meldunge geschrieben werden sollen.
	 */
	private $debugEnvironments;
	private $debuggerVoid;
	private $debuggerEchoLinebreak;
	private $debuggerEchoHTML;
	private $debuggerEchoFile;
	public $outputOnEndString;

	public function __construct() {
		$this->debugEnvironments = array();

		$this->debuggerVoid = new DebuggerVoid();
		$this->debuggerEchoLinebreak = new DebuggerEchoLinebreak();
		$this->debuggerEchoHTML = new DebuggerEchoHTML();
		$this->debuggerEchoFile = new DebuggerFile();

		$this->outputOnEndString = "";
	}

	// Fehler ausgeben, falls keine Config Variable eingetragen wurde
	public function addDebugEnvironment($environmentName,$debugCount,$showDebugWithPRE,$showDebugCategories,$showDebugCount,$showDebugMessagesOnEnd,$usedDebuggers) {
		$this->debugEnvironments[$environmentName] =
		new DebugEnvironmentObject($environmentName,$debugCount,$showDebugWithPRE,$showDebugCategories,$showDebugCount,$showDebugMessagesOnEnd,$usedDebuggers);
	}

	public function showEnvironmentDebugMessage ($message, $debugCategorie, $debugCount) {
		$tmpMessage = $message;
		if ($this->debugEnvironments[$debugCategorie]->showDebugCategories) {
			if ($this->debugEnvironments[$debugCategorie]->showDebugCount) {
				$tmpMessage = "[". $debugCategorie ."[". $debugCount ."]] ". $tmpMessage;
			}
			else {
				$tmpMessage = "[". $debugCategorie ."] ". $tmpMessage;
			}
		}

		// echo "call showEnvironmentDebugMessage<br>";
		// is debugEnvironment set up?
		if (isset ($this->debugEnvironments[$debugCategorie])) {
			// is debugCount high enough?
			if ($this->debugEnvironments[$debugCategorie]->debugCount >= $debugCount) {
				// show all debug Message on end of website?
				// show all debug Message on end of website?
				if ($this->debugEnvironments[$debugCategorie]->showDebugMessagesOnEnd) {
					$this->outputOnEndString .= $tmpMessage . "\n";
					return;
				}

				// DEBUG_MODE_ECHO_LINEBREAK
				if ($this->debugEnvironments[$debugCategorie]->usedDebuggers % DEBUG_MODE_ECHO_LINEBREAK == 0) {
					// debug with PRE?
					if (!$this->debugEnvironments[$debugCategorie]->showDebugWithPRE) {
						$this->debuggerEchoLinebreak->showDebugMessage($tmpMessage);
					}
					else {
						$this->debuggerEchoLinebreak->showDebugMessage("<PRE>" . $tmpMessage . "</PRE>");
					}
				}

				// DEBUG_MODE_ECHO_HTML
				if ($this->debugEnvironments[$debugCategorie]->usedDebuggers % DEBUG_MODE_ECHO_HTML == 0) {
					// debug with PRE?
					if (!$this->debugEnvironments[$debugCategorie]->showDebugWithPRE) {
						$this->debuggerEchoHTML->showDebugMessage($tmpMessage);
					}
					else {
						$this->debuggerEchoLinebreak->showDebugMessage("<PRE>" . $tmpMessage . "</PRE>");
					}

					// DEBUG_MODE_ECHO_FILE
					if ($this->debugEnvironments[$debugCategorie]->usedDebuggers % DEBUG_MODE_FILE == 0) {
						$this->debuggerFile->showDebugMessage($tmpMessage);
					}
				}
			} // is debugCount high enought
		} // is debugEnvironment set up
		else {
			echo 'NOTICE: debugEnvironment $this->debugEnvironments["'. $debugCategorie .'"] did not exists <br>'. "\n";
		}

	}// end from function showDebugMessage

	public function showDebugMessage ($message) {
		// Dummy Methode, da tatsaechliche Ausgaben ueber showEnvironmentDebugMessage ausgegeben werden sollten
		// TODO: EXCEPTION werfen
		echo 'DEPRECATED CALL showDebugMessages($message)<br>';
	}

	public function __toString() {
		$returnString = "[class environmentDebugObject]\n";
		foreach ($this->debugEnvironments as $key => $value) {
			$returnString .= "   ". $value->environmentName . "\n";
			$returnString .= "      debugCount:". $value->debugCount . "\n";
			$returnString .= "      showDebugWithPRE: ". $value->showDebugWithPRE . "\n";
			$returnString .= "      showDebugMessagesOnEnd: ". $value->showDebugMessagesOnEnd . "\n";
			// $returnString .= "      usedDebuggers: ". $value->usedDebuggers . "\n";
			$tmpUsedDebuggers = $value->usedDebuggers;
			$returnString .= "      usedDebuggers: ";
			if (($tmpUsedDebuggers != 0) && ($tmpUsedDebuggers % DEBUG_MODE_VOID == 0)) {
				$returnString 		.= '"DEBUG_MODE_VOID" ';
				$tmpUsedDebuggers 	= $tmpUsedDebuggers / DEBUG_MODE_VOID;
				if ($tmpUsedDebuggers == 1) {$tmpUsedDebuggers = 0;}
			}
			if (($tmpUsedDebuggers != 0) && ($tmpUsedDebuggers % DEBUG_MODE_ECHO_LINEBREAK == 0)) {
				$returnString 		.= '"DEBUG_MODE_ECHO_LINEBREAK" ';
				$tmpUsedDebuggers 	= $tmpUsedDebuggers / DEBUG_MODE_ECHO_LINEBREAK;
				if ($tmpUsedDebuggers == 1) {$tmpUsedDebuggers = 0;}
			}
			if (($tmpUsedDebuggers != 0) && ($tmpUsedDebuggers % DEBUG_MODE_ECHO_HTML == 0)) {
				$returnString 		.= '"DEBUG_MODE_ECHO_HTML" ';
				$tmpUsedDebuggers 	= $tmpUsedDebuggers / DEBUG_MODE_ECHO_HTML;
				if ($tmpUsedDebuggers == 1) {$tmpUsedDebuggers = 0;}
			}
			if (($tmpUsedDebuggers != 0) && ($tmpUsedDebuggers % DEBUG_MODE_FILE == 0)) {
				$returnString 		.= '"DEBUG_MODE_FILE" ';
				$tmpUsedDebuggers 	= $tmpUsedDebuggers / DEBUG_MODE_FILE;
				if ($tmpUsedDebuggers == 1) {$tmpUsedDebuggers = 0;}
			}
			/*
			 define ('DEBUG_MODE_VOID'			,2);
			 define ('DEBUG_MODE_ECHO_LINEBREAK'	,4);
			 define ('DEBUG_MODE_ECHO_HTML'		,8);
			 define ('DEBUG_MODE_FILE'			,16);
			 */
			if ($tmpUsedDebuggers != 0) {$returnString .= '"UNDEFINED USED DEBUGGER: '. $tmpUsedDebuggers . '"';}
			// $returnString .= "\n";
		}
		return $returnString;

	}
	function __destruct() {
		if ($this->outputOnEndString != "") {
			echo "<PRE>outputOnEndString:\n". $this->outputOnEndString ."</PRE>";
			}
		/*
		 foreach ($this->debugEnvironments as $key => $value) {
		 if ($value->showDebugMessagesOnEnd) {

		 }
		 $returnString .= "      showDebugMessagesOnEnd: ".  . "\n";
		 $returnString .= "   ". $value->environmentName . "\n";
		 */
	}

}

