<?

namespace de\IPSymcon\ernie;

/**********************************************
 ***
 *** INTERFACE Heizungskomponenten
 ***
 ***/
interface HeizungsKomponenten {
	// Heizungs Funktionen
	public function __construct($varConfigObject);
	public function __toString();
	public function getSollTemperatur ($raumName);
	public function setSollTemperatur ($raumName,$sollTemperatur,$grund);
	public function setHeizenJaNein ($raumName,$heizenJaNein,$sollTemperatur,$grund);
	public function getWindowStatus ($objectID);
	
	// Fenster Stati-Funktionen
	public function getOpendWindowValue();
	public function getTitedWindowValue();
	public function getClosedWindowValue();
	
	// implementierte Heizungssteuerungen
	public function isControlImplemented($controlName);
}

/***************************************************************
 ***
 *** STANDARD IMPLEMENTATION OF INTERFACE Heizungskomponenten
 ***
 ***/
class StandardHeizungsKomponenten implements HeizungsKomponenten {
	protected $configObject;
	protected $implementedControls;

	// Heizungs Funktionen
	public function __construct($varConfigObject) {}

	public function getSollTemperatur ($raumName) {}
	public function setSollTemperatur ($raumName,$sollTemperatur,$grund) {}
	public function setHeizenJaNein ($raumName,$heizenJaNein,$sollTemperatur,$grund){}
	public function getWindowStatus ($objectID) {}

	// Fenster Stati-Funktionen
	public function getOpendWindowValue() {}
	public function getTitedWindowValue() {}
	public function getClosedWindowValue() {}

	// implementierte Heizungssteuerungen
	public function isControlImplemented($controlName) {
		// echo '$this->implementedControls["SIMPLE-DELTA"] == ' . $this->implementedControls['SIMPLE-DELTA'] . "\n";
		// echo get_class($this) ."\n";
		return $this->implementedControls[$controlName];
		}

	public function __toString() {return get_class($this);}

	}

/*******************************************************
 ***
 *** IMPLEMENTATIONS OF INTERFACE Heizungskomponenten
 ***
 ***/
class SendeHeizenFloatTemperatur extends StandardHeizungsKomponenten {
	protected $heizungsKomponenten;

	public function __construct($varConfigObject) {
		$this->configObject = $varConfigObject;
		$this->heizungsKomponenten = $this->configObject->getHeizungsKomponenten();
		
		$this->implementedControls = array();
		$this->implementedControls['KEINE'] 					= TRUE;
		$this->implementedControls['SIMPLE-DELTA'] 			= TRUE;
		$this->implementedControls['SOLL-IST-VERGLEICH'] 	= FALSE;
		$this->implementedControls['BERSCHWAPPT-DELTA'] 	= FALSE;
		// echo 'construct: $this->implementedControls["SIMPLE-DELTA"] == ' . $this->implementedControls['SIMPLE-DELTA'] . "\n";
		$this->configObject->showEnvironmentDebugMessage("Object SendeHeizenFloatTemperatur erstellt" ,"SendeHeizenFloatTemperatur.class[__construct]",1);
	}

	public function getSollTemperatur ($raumName) {
		$this->configObject->showEnvironmentDebugMessage('call getSollTemperatur("'. $raumName.'")' ,"SendeHeizenFloatTemperatur.class[getSollTemperatur]",1);
	   $setPointID = $this->configObject->getReadRaumSollTemperaturObjectID($raumName);
		$this->configObject->showEnvironmentDebugMessage('   $setPointID == '. $setPointID ,"SendeHeizenFloatTemperatur.class[getSollTemperatur]",1);
		$result = GetValueFloat($setPointID);
		$this->configObject->showEnvironmentDebugMessage('   $result == '. $result ,"SendeHeizenFloatTemperatur.class[getSollTemperatur]",1);
		return $result ;
	}

	public function setSollTemperatur ($raumName,$wunschTemperatur,$grund) {
	   $setPointID = $this->configObject->getWriteRaumSollTemperaturObjectID($raumName);
	   $result = "-1";
		if (heizungsKomponenten == "HomeMatic") {$result = HM_WriteValueFloat($setPointID, "SETPOINT", $wunschTemperatur);}
		if (heizungsKomponenten == "FS20") {$result = FHT_SetTemperature($setPointID, $wunschTemperatur);}
		if ($result) {
		   IPS_LogMessage("SendeHeizenFloatTemperatur.class","Temperatur fr Heizkrper ". $raumName ." auf " . $wunschTemperatur . " gesetzt; Grund: ". $grund);
		   }
		else {
		   IPS_LogMessage("SendeHeizenFloatTemperatur.class","MISSLUNGENER Versuch, Temperatur fr Heizkrper ". $raumName ." auf " . $wunschTemperatur . " zu setzen");
		}
		return $result;
	}

	public function getWindowStatus ($objectID) {return GetValueInteger($objectID);}

	// Fenster Stati-Funktionen fr HomeMatic
	public function getOpendWindowValue() {return 2;} // $FENSTER_OFFEN = 2;

	public function getTitedWindowValue() {return 1;} // $FENSTER_GEKIPPT = 1;

	public function getClosedWindowValue() {return 0;}// $FENSTER_GESCHLOSSEN = 0;
}

/********************************************************
 ***
 *** IMPLEMENTATIONS OF INTERFACE Heizungskomponenten
 ***
 ***/
class SendeHeizenBooleanJaNein extends StandardHeizungsKomponenten {
	public function __construct($varConfigObject) {
		$this->configObject = $varConfigObject;

		$this->implementedControls['KEINE'] 					= FALSE;
		$this->implementedControls['SIMPLE-DELTA'] 			= FALSE;
		$this->implementedControls['SOLL-IST-VERGLEICH'] 	= TRUE;
		$this->implementedControls['BERSCHWAPPT-DELTA'] 	= FALSE;
		$this->configObject->showEnvironmentDebugMessage("Object SendeHeizenBooleanJaNein erstellt" ,"SendeHeizenBooleanJaNein.class[__construct]",1);
	}

	public function getSollTemperatur ($raumName) {
		$this->configObject->showEnvironmentDebugMessage('call getSollTemperatur("'. $raumName.'")' ,"SendeHeizenBooleanJaNein.class[getSollTemperatur]",1);
	   $setPointID = $this->configObject->getReadRaumSollTemperaturObjectID($raumName);
		$this->configObject->showEnvironmentDebugMessage('   $setPointID == '. $setPointID ,"SendeHeizenBooleanJaNein.class[getSollTemperatur]",1);
		$result = GetValueFloat($setPointID);
		$this->configObject->showEnvironmentDebugMessage('   $result == '. $result ,"SendeHeizenBooleanJaNein.class[getSollTemperatur]",1);
		return $result ;
	}

	public function setHeizenJaNein ($raumName,$heizenJaNein,$sollTemperatur,$grund){
	   // $antriebsID = $this->configObject->getWriteRaumSollTemperaturObjectID($raumName);
	   $sollTemperaturObjectID = $this->configObject->getWriteRaumSollTemperaturObjectID($raumName);

		$heizenJaNeinString = "ABKHLEN";
		if ($heizenJaNein) {
		   $heizenJaNeinString = "HEIZEN";
		   }

	   // 1.) Stellabtrieb setzen
	   // Der Stellantrieb muss nicht umbedingt gesetzt werden,
		// da die WAGO auf das ndern der HeizStati-Variable reagiert
		// $result = SetValueBoolean($antriebsID, $heizenJaNein);
	   IPS_LogMessage("SendeHeizenBooleanJaNein.class","Temperatur fr Heizkrper ". $raumName ." auf ". $heizenJaNeinString." gesetzt; Grund: ". $grund);

		// 2.) SOLL-Temperatur setzen
		SetValueFloat($sollTemperaturObjectID,$sollTemperatur);

		// 3.) Raum-Heizstatus-Variable Heizen/Abkuehlen setzen
		$objectIDRaumHeizStatus = $this->configObject->getObjectIDRaumHeizStatus($raumName);
		SetValueBoolean($objectIDRaumHeizStatus,$heizenJaNein);

		return 0;
	}

	public function getWindowStatus ($objectID) {return GetValueInteger($objectID);}

	// Fenster Stati-Funktionen fr 1-Wire bzw. Wago
	public function getOpendWindowValue() {return 2;} // $FENSTER_OFFEN = 2;

	public function getTitedWindowValue() {return 1;} // $FENSTER_GEKIPPT = 1;

	public function getClosedWindowValue() {return 0;}// $FENSTER_GESCHLOSSEN = 0;
}

/******************************************************
 ***
 *** IMPLEMENTATIONS OF INTERFACE Heizungskomponenten
 ***
 ***/
class SendeHeizenFloatTemperatur6und30Grad extends StandardHeizungsKomponenten {
	public function __construct($varConfigObject) {
		$this->configObject = $varConfigObject;

		$this->implementedControls = array();
		$this->implementedControls['KEINE'] 					= FALSE;
		$this->implementedControls['SIMPLE-DELTA'] 			= FALSE;
		$this->implementedControls['SOLL-IST-VERGLEICH'] 	= TRUE;
		$this->implementedControls['BERSCHWAPPT-DELTA'] 	= FALSE;
		// echo 'construct: $this->implementedControls["SIMPLE-DELTA"] == ' . $this->implementedControls['SIMPLE-DELTA'] . "\n";
		$this->configObject->showEnvironmentDebugMessage("Object SendeHeizenFloatTemperatur6und30Grad erstellt" ,"SendeHeizenFloatTemperatur6und30Grad.class[__construct]",1);
	}

	public function getSollTemperatur ($raumName) {
		$this->configObject->showEnvironmentDebugMessage('call getSollTemperatur("'. $raumName.'")' ,"SendeHeizenFloatTemperatur6und30Grad.class[getSollTemperatur]",1);
	   $setPointID = $this->configObject->getReadRaumSollTemperaturObjectID($raumName);
		$this->configObject->showEnvironmentDebugMessage('   $setPointID == '. $setPointID ,"SendeHeizenFloatTemperatur6und30Grad.class[getSollTemperatur]",1);
		$result = GetValueFloat($setPointID);
		$this->configObject->showEnvironmentDebugMessage('   $result == '. $result ,"SendeHeizenFloatTemperatur6und30Grad.class[getSollTemperatur]",1);
		return $result ;
	}

	public function setHeizenJaNein ($raumName,$heizenJaNein,$sollTemperatur,$grund){
	// public function setSollTemperatur ($raumName,$wunschTemperatur,$grund) {
		// setPoint wird hier nur zum setzen der 6 bzw. 30 verwendet,
		// die ReadRaumTemperatur ist eigentliche SOLL-Temperatur
		//
	   $homeMaticSetPointObjectID = $this->configObject->getWriteRaumSollTemperaturObjectID($raumName);
	   $sollTemperaturObjectID = $this->configObject->getReadRaumSollTemperaturObjectID($raumName);

		$homeMaticSollTemperatur = 6;
		$heizenJaNeinString = "ABKHLEN";
		if ($heizenJaNein) {
		   $homeMaticSollTemperatur = 30;
		   $heizenJaNeinString = "HEIZEN";
		   }

		$this->configObject->showEnvironmentDebugMessage('$homeMaticSollTemperatur == '. $homeMaticSollTemperatur,"SendeHeizenFloatTemperatur6und30Grad.class[setHeizenJaNein]",9);

	   // 1.) HomeMatic auf 6 oder 30 setzen
		$result = HM_WriteValueFloat($homeMaticSetPointObjectID, "SETPOINT",$homeMaticSollTemperatur);
		if ($result) {
		   IPS_LogMessage("SendeHeizenFloatTemperatur6und30Grad.class","Temperatur fr Heizkrper ". $raumName ." auf ". $heizenJaNeinString." gesetzt; Grund: ". $grund);
			// 2.) SOLL-Temperatur setzen
			SetValueFloat($sollTemperaturObjectID,$sollTemperatur);

			// 3.) Raum-Heizstatus-Variable Heizen/Abkuehlen setzen
			$objectIDRaumHeizStatus = $this->configObject->getObjectIDRaumHeizStatus($raumName);
			SetValueBoolean($objectIDRaumHeizStatus,$heizenJaNein);
			}
		else {
		   IPS_LogMessage("SendeHeizenFloatTemperatur6und30Grad.class","MISSLUNGENER Versuch, Temperatur fr Heizkrper ". $raumName ." auf ". $heizenJaNeinString ." zu setzen");
		}

		return $result;
	}

	public function getWindowStatus ($objectID) {return GetValueInteger($objectID);}

	// Fenster Stati-Funktionen fr HomeMatic
	public function getOpendWindowValue() {return 2;} // $FENSTER_OFFEN = 2;

	public function getTitedWindowValue() {return 1;} // $FENSTER_GEKIPPT = 1;

	public function getClosedWindowValue() {return 0;}// $FENSTER_GESCHLOSSEN = 0;
}

?>