<?

namespace de\IPSymcon\ernie;

class HeizungsTrigger {
	private $configObject;
	private $heizungsKomponenten;

	public function __construct($varConfigObject) {
		$this->configObject = $varConfigObject;
		$this->configObject->showEnvironmentDebugMessage("Object HeizungsTrigger erstellt" ,"heizungssteuerung_trigger.class[__construct]",1);
		
		$this->heizungsKomponenten = "";
		if ($this->configObject->getHeizungsInterface() == "sendeHeizenFloatTemperatur") {
			   if ($this->configObject->getHeizungsKomponenten() == "") {
				IPS_LogMessage("trigger.class","FEHLER: heizungssteuerung['heizungsKomponenten'] darf nicht leer sein");
				die();
				}
		   $this->heizungsKomponenten = new SendeHeizenFloatTemperatur($varConfigObject);
		   }
		if ($this->configObject->getHeizungsInterface() == "sendeHeizenFloatTemperatur6und30Grad") {
		   $this->heizungsKomponenten = new sendeHeizenFloatTemperatur6und30Grad($varConfigObject);
		   }
		if ($this->configObject->getHeizungsInterface() == "sendeHeizenBooleanJaNein") {
		   $this->heizungsKomponenten = new SendeHeizenBooleanJaNein($varConfigObject);
			}
		if ($this->heizungsKomponenten == "") {
			IPS_LogMessage("trigger.class","FEHLER: Heizungs-Komponenten-Object konnte nicht erstellt werden (". $this->configObject->getHeizungsKomponenten() .")");
			die();
			}
	}

	public function initialize() {
		// rootNode des heutigen Tages raussuchen:
		$actualWeekDay = date("N");
		$this->configObject->showEnvironmentDebugMessage('$actualWeekDay == '. $actualWeekDay ,"heizungssteuerung_trigger.class[showActualHeizplan]",9);

		// $actualWeekDay = 0; // statisch angelegter Test-Tag
		// $this->configObject->showEnvironmentDebugMessage('set $actualWeekDay to '. $actualWeekDay ,"heizungssteuerung_trigger.class[showActualHeizplan]",9);

		$todayRootNode = $this->configObject->GetRootNodeForDay($actualWeekDay);
		$this->configObject->showEnvironmentDebugMessage('$todayRootNode == '. $actualWeekDay ,"heizungssteuerung_trigger.class[showActualHeizplan]",9);
		$now = time();
		$actualHour = date("G");
		$actualMinute = date("i");
		$this->configObject->showEnvironmentDebugMessage('aktuelle Uhrzeit: '. $actualHour .":". $actualMinute ,"heizungssteuerung_trigger.class[showActualHeizplan]",9);

		// heutigen Heizplan bzw. aktuelle SOLL-Temperaturen auslesen
		$childrenIDsToday = IPS_GetChildrenIDs($todayRootNode);
		$this->configObject->showEnvironmentDebugMessage("Heutiger Heizplan:","heizungssteuerung_trigger.class[showActualHeizplan]",8);

		// merken, welche Rume berhaupt vorhanden sind, damit Rume die fr die kein aktuelles
		// Zeitprofil vorhanden ist, trotzdem am Ende abgesenkt werden knnen
		$vorhandeneRaeume = array();

		// merken, fr welche Rume ich bereits eine Absenktemperatur durch ein Offenens/Gekipttes Fenster gesetzt habe
		// und diese bei der normalen Temperatur nderunge auf Zeit nicht mehr bercksichtigen!
		$offeneFensterVeraenderteRaeume = array();
		
		// merken, welche Rume durch ein Zeitprofil gesetzt wurden
		$zeitprofilVeraenderteRaeume = array();

		// Alle Heizplan Variablen durchgehen
		for ($i = 0; $i < count($childrenIDsToday); $i++) {
		   // die Variablen werden nach ObjectIDs durchgegangen, also UNSORTIERT
			$actualVariableName = IPS_GetName($childrenIDsToday[$i]);
			$this->configObject->showEnvironmentDebugMessage("   ". $actualVariableName ,"heizungssteuerung_trigger.class[showActualHeizplan]",9);

			// Raum und Zeit bestimmen
			$actualRaumUndZeit = explode("-",$actualVariableName);
			$actualRaum = trim($actualRaumUndZeit[0]);
			$actualZeitString = trim($actualRaumUndZeit[1]);
			$actualZeitName = "";
			$actualHeizplanConfigConstantsArray = $this->configObject->getCreateHeizplanConfigConstantsArray();
			if (!$actualHeizplanConfigConstantsArray['CreateShortHeizplanVariablen']) {
				$actualZeitName = trim($actualRaumUndZeit[2]);
				}

			$actualZeit = explode(" ",$actualZeitString);
			$actualZeitStart = explode(":",$actualZeit[0]);
			$actualZeitStartHour = $actualZeitStart[0];
			$actualZeitStartMinute = $actualZeitStart[1];
			$actualZeitStop = explode(":",$actualZeit[2]);
			$actualZeitStopHour = $actualZeitStop[0];
			$actualZeitStopMinute = $actualZeitStop[1];
			
			$timeStart = mktime($actualZeitStartHour,$actualZeitStartMinute);
			$timeStop = mktime($actualZeitStopHour,$actualZeitStopMinute);

			$this->configObject->showEnvironmentDebugMessage('$actualRaum == '. $actualRaum,"heizungssteuerung_trigger.class[showActualHeizplan]",8);
			$actualZeitOutput = $actualZeitStartHour .":". $actualZeitStartMinute ." / ". $actualZeitStopHour .":". $actualZeitStopMinute;
			$this->configObject->showEnvironmentDebugMessage('   $actualZeit == '. $actualZeitOutput,"heizungssteuerung_trigger.class[showActualHeizplan]",8);
			$this->configObject->showEnvironmentDebugMessage('   $actualZeitName == '. $actualZeitName,"heizungssteuerung_trigger.class[showActualHeizplan]",8);
			$this->configObject->showEnvironmentDebugMessage('   ',"heizungssteuerung_trigger.class[showActualHeizplan]",8);

			$this->configObject->showEnvironmentDebugMessage('$now = '. $now ." (". date(DATE_RFC822,$now) .")","heizungssteuerung_trigger.class[showActualHeizplan]",8);
			$this->configObject->showEnvironmentDebugMessage('$timeStart = '. $timeStart ." (". date(DATE_RFC822,$timeStart) .")","heizungssteuerung_trigger.class[showActualHeizplan]",8);
			$this->configObject->showEnvironmentDebugMessage('$timeStop = '. $timeStop." (". date(DATE_RFC822,$timeStop) .")","heizungssteuerung_trigger.class[showActualHeizplan]",8);

			// aktuellen Raum in den Vorhandenen Rumen merken
			$vorhandeneRaeume[$actualRaum] = TRUE;

			// #############################################################
			// ###
			// ### ist fr den aktuellen Raum ein Fenster vorhanden?
			// ### wenn dieses Geffnet oder Gekippt ist
			// ### ==> Temperaturnderung auf Abwesend-Temperatur
			// ###
			// echo ('$offeneFensterVeraenderteRaeume == ');
			// print_r($offeneFensterVeraenderteRaeume);

		   // wurde fr den aktuellen Raum schon das Array fr offeneFenster gesetzt?
			if (!isset($offeneFensterVeraenderteRaeume[$actualRaum])) {
				// OffeneFensternderung fr diesen Raum als Geschlossen definieren
				// (wird eventuell weiter unten mit "Offen" berschrieben)
				$offeneFensterVeraenderteRaeume[$actualRaum] = FALSE;
				// $this->configObject->showEnvironmentDebugMessage('set $offeneFensterVeraenderteRaeume['.$actualRaum .'] = 0',
				//	"heizungssteuerung_trigger.class[lookOpenWindows]",9);
			}

			$actualWindowsArray = $this->configObject->getWindowRootNodeForRoom($actualRaum);
			// sind berhaupt Fenster fr den aktuellen Raum vorhanden?
			if ($actualWindowsArray != -1) {
				$this->configObject->showEnvironmentDebugMessage('Anzahl der Fenster im '.$actualRaum .' == '. count($actualWindowsArray),
					"heizungssteuerung_trigger.class[lookOpenWindows]",9);

				for ($j = 0; $j <= count($actualWindowsArray) - 1; $j++) {
					// wurde die Temperatur fr diesen Raum aufgrund eines Offenen/Gekiptten Fenster schon herabgesetzt?
					if ($offeneFensterVeraenderteRaeume[$actualRaum] == FALSE) {
						$this->configObject->showEnvironmentDebugMessage('   ObjectID $actualWindowArray'. $actualRaum .'['. $j .'] == '. $actualWindowsArray[$j],
							"heizungssteuerung_trigger.class[lookOpenWindows]",9);

						$actualWindowsStatus = $this->heizungsKomponenten->getWindowStatus($actualWindowsArray[$j]);
						// Fenster testweise statisch setzen:
						// $actualWindowsStatus = 2;

						$actualWindowsStatusString = "";
						if ($actualWindowsStatus == $this->heizungsKomponenten->getClosedWindowValue()) {$actualWindowsStatusString = "Fenster Geschlossen";}
						if ($actualWindowsStatus == $this->heizungsKomponenten->getOpendWindowValue()) {$actualWindowsStatusString = "Fenster Offen";}
						if ($actualWindowsStatus == $this->heizungsKomponenten->getTitedWindowValue()) {$actualWindowsStatusString = "Fenster Gekippt";}
						$this->configObject->showEnvironmentDebugMessage('   Status Fenster '. $j .' im '.$actualRaum .' == '. $actualWindowsStatusString,
							"heizungssteuerung_trigger.class[lookOpenWindows]",9);

						// ist das Fenster offen oder gekippt?
						if ($actualWindowsStatus == $this->heizungsKomponenten->getOpendWindowValue()
							|| $actualWindowsStatus == $this->heizungsKomponenten->getTitedWindowValue()) {
						   // Absenktemperatur einstellen
							$this->configObject->showEnvironmentDebugMessage('   ==> Absenktemperatur einstellen',"heizungssteuerung_trigger.class[lookOpenWindows]",9);

							$actualAnwesenheit = FALSE;

							// merken, dass fr diesen Raum die Temperatur bereits durch ein Offenes/Gekipttes Fenster abgesenkt wurde oder nun wird:
							$this->configObject->showEnvironmentDebugMessage('      $offeneFensterVeraenderteRaeume['. $actualRaum .'] set to TRUE',
								"heizungssteuerung_trigger.class[lookOpenWindows]",7);
							$offeneFensterVeraenderteRaeume[$actualRaum] = TRUE;
							
							$grund = 'Temperaturnderung durch Offenens/Gekipptes Fenster auf Abwesenheits-Temperatur herabgesetzt';
							$this->setRaumTemperatur($actualRaum,$actualAnwesenheit,$grund);
						} // ist das Fenster offen oder gekippt?
					} // wurde die Temperatur fr diesen Raum aufgrund eines Offenen/Gekiptten Fenster schon herabgesetzt?
					else { // von if ($actualWindowsArray != -1) { // sind berhaupt Fenster fr den aktuellen Raum vorhanden?
						$this->configObject->showEnvironmentDebugMessage('Temperatur im '. $actualRaum .' wurde bereits aufgrund eines Offenen/Gekippten Fensters herabgesetzt!',
							"heizungssteuerung_trigger.class[lookOpenWindows]",9);
					} // ELSE von "wurde die Temperatur fr diesen Raum aufgrund eines Offenen/Gekiptten Fenster schon herabgesetzt?"
				} // for ($j =
			} // if ($actualWindowsArray != -1) {
			else { // von 	

				$this->configObject->showEnvironmentDebugMessage('KEINE Fenster im '.$actualRaum .' vorhanden!',
					"heizungssteuerung_trigger.class[lookOpenWindows]",9);
			}

			// ########################################################################
			// ###
			// ### Zeit-Profile auslesen
			// ###
			// ### liegt die aktuelle Uhrzeit innerhalb der aktuellen Heizvariable?
			// ### ==> Temperaturnderung auf aktuelle An-/Abwesend-Temperatur
			// ###

		   // wurde fr den aktuellen Raum schon das Array fr offeneFenster gesetzt?
			if (!isset($zeitprofilVeraenderteRaeume[$actualRaum])) {
			   // ZeitProfilnderung fr diesen Raum als nicht vorhanden definieren
				// (wird eventuell weiter unten mit "Vorhanden" berschrieben)
				$zeitprofilVeraenderteRaeume[$actualRaum] = FALSE;
				// $this->configObject->showEnvironmentDebugMessage('set $zeitprofilVeraenderteRaeume['.$actualRaum .'] = 0',
				//	"heizungssteuerung_trigger.class[showActualHeizplan]",7);
			}

			if ($now > $timeStart && $now < $timeStop) {
				// echo ('$offeneFensterVeraenderteRaeume(2) == ');
				// print_r($offeneFensterVeraenderteRaeume);
				// $this->configObject->showEnvironmentDebugMessage('$offeneFensterVeraenderteRaeume['. $actualRaum .'] == '. $offeneFensterVeraenderteRaeume[$actualRaum],
				//	"heizungssteuerung_trigger.class[showActualHeizplan]",7);

				// wurde die Temperatur fr diesen Raum aufgrund eines Offenen/Gekiptten Fenster schon herabgesetzt?
				if ($offeneFensterVeraenderteRaeume[$actualRaum] == FALSE) {
					$this->configObject->showEnvironmentDebugMessage(
						date("H:i",$now) .' liegt innerhalb des Heizplans "'. $actualRaum .'" von "'. $actualZeitString ." (". $actualZeitName
						.') " ==> Temperatur eventuell anpassen ...',"heizungssteuerung_trigger.class[showActualHeizplan]",7);

					// aktuelle Anwesenheit auslesen
					$actualVariableObject = IPS_GetVariable($childrenIDsToday[$i]);
					$actualVariableValueArray = $actualVariableObject['VariableValue'];
					// Boolean ist entweder leer oder enthllt 1 als Inhalt
					// print_r($actualVariableObject);
					$actualAnwesenheit = $actualVariableValueArray['ValueBoolean'];
					$this->configObject->showEnvironmentDebugMessage("Anwesenheit ". $actualRaum ." == ". $actualAnwesenheit,"heizungssteuerung_trigger.class[setActualTemperature]",8);

					$grund = 'Temperaturnderung durch An/Abwesenheit im Zeitprofil ausgelst';
					$this->setRaumTemperatur($actualRaum,$actualAnwesenheit,$grund);
					
					$zeitprofilVeraenderteRaeume[$actualRaum] = TRUE;

				} // wurde die Temperatur fr diesen Raum aufgrund eines Offenen/Gekiptten Fenster schon herabgesetzt? / if (!isset($offeneFensterVeraenderteRaeume[$actualRaum])) {
				else {
					$this->configObject->showEnvironmentDebugMessage('   Temperatur im '. $actualRaum .' wurde bereits aufgrund eines Offenen/Gekippten Fensters herabgesetzt!',
						"heizungssteuerung_trigger.class[setActualTemperature]",7);
				} // ELSE von wurde die Temperatur fr diesen Raum aufgrund eines Offenen/Gekiptten Fenster schon herabgesetzt? / if (!isset($verarbeiteteRaeume[$actualRaum])) {
			} // liegt die aktuelle Uhrzeit innerhalb der aktuellen Heizvariable? / if ($now > $timeStart && $now < $timeStop) {
		} // Alle Heizplan Variablen durchgehen / for ($i = 0; $i < count($childrenIDsToday); $i++) {

	// ##########################################################################
	// ###
	// ### Am Ende noch schauen, ob fr jeden Raum eine Temperatur gesetzt wurde?
	// ### Entweder durch ein offenes/gekipttes Fenster oder durch ein ZeitProfil
	//
	// ### Grund: Wenn fr einen Raum die Heizung tatschlich
	// ### am Ende vom Tag bis zum Ende des Zeitprofils laufen soll,
	// ### wrde Sie die ganze Nacht durchlaufen, da kein Zeitprofil Sie
	// ### auf die "Abwesend-"Temperatur zurcksetzt

	// print_r($vorhandeneRaeume);
	foreach ($vorhandeneRaeume as $key => $value) {
	   $actualRaum = $key;
	   
		// echo '$offeneFensterVeraenderteRaeume == ';
		// print_r($offeneFensterVeraenderteRaeume);
	   $offenesFensterAenderung = $offeneFensterVeraenderteRaeume[$actualRaum];
	   // if ($offenesFensterAenderung == FALSE) {$offenesFensterAenderung = 0;}

	   // echo '$zeitprofilVeraenderteRaeume == ' . "\n";
		// print_r($zeitprofilVeraenderteRaeume);
	   $zeitprofilAenderung = $zeitprofilVeraenderteRaeume[$actualRaum];
	   // if ($zeitprofilAenderung == FALSE) {$zeitprofilAenderung = 0;}

		$this->configObject->showEnvironmentDebugMessage('aktueller Raum == '. $key,"heizungssteuerung_trigger.class[lookAllRooms]",9);
		$this->configObject->showEnvironmentDebugMessage('   $offenesFensterAenderung == '. $offenesFensterAenderung,"heizungssteuerung_trigger.class[lookAllRooms]",9);
		$this->configObject->showEnvironmentDebugMessage('   $zeitprofilAenderung == '. $zeitprofilAenderung,"heizungssteuerung_trigger.class[lookAllRooms]",9);

		// wenn weder ein Zeitprofil noch ein offenen/gekipptes Fenster fr den Raum vorhanden ist
		// wird der Raum auf Abwesend-Temperatur gendert
		if (!$offenesFensterAenderung && !$zeitprofilAenderung) {
			$actualAnwesenheit = FALSE;
	
			$grund = 'Temperatur durch kein aktuelles Zeitprofil / kein offenes oder gekipptes Fenster auf Abwesenheits-Temperatur abgesenkt';
			$this->setRaumTemperatur($actualRaum,$actualAnwesenheit,$grund);

			$this->configObject->showEnvironmentDebugMessage($actualRaum .': '. $grund,"heizungssteuerung_trigger.class[lookAllRooms]",8);
		   }
		}

	} // ende von function initialize()

	public function setRaumTemperatur($actualRaum,$actualAnwesenheit,$grund) {
	
		$actualSollTemperatur = $this->heizungsKomponenten->getSollTemperatur($actualRaum);
		// SollTemperatur testweise statisch setzen:
		// $actualSollTemperatur = 20;
		$this->configObject->showEnvironmentDebugMessage(
			'aktuelle Soll-Temperatur "'. $actualRaum .'" bertrgt '. $actualSollTemperatur ." Grad","heizungssteuerung_trigger.class[setRaumTemperatur]",9);

		$actualWunschTemperatur = $this->configObject->getWunschTemperatur($actualRaum,$actualAnwesenheit);
		$this->configObject->showEnvironmentDebugMessage('$actualWunschTemperatur == '. $actualWunschTemperatur,"heizungssteuerung_trigger.class[setRaumTemperatur]",9);

		$raumSteuerungsTechnik = $this->configObject->getRaumSteuerungsTechnik($actualRaum);

	   // ist die aktuelle RaumSteuerungsTechnik fr das aktuelle Interface implementiert?
	   //
	   $raumSteuerungsTechnikDefiniert = $this->heizungsKomponenten->isControlImplemented($raumSteuerungsTechnik);
	   $debugMessage = "FEHLER: ". $this->heizungsKomponenten . " implementiert NICHT die Raum-Steuerungs-Technik  ". $raumSteuerungsTechnik ." (Raum: ". $actualRaum .")";
		$this->configObject->showEnvironmentDebugMessage('$raumSteuerungsTechnikDefiniert == '. $raumSteuerungsTechnikDefiniert ."; ". $actualRaum ,"heizungssteuerung_trigger.class[setRaumTemperatur]",5);
		$this->configObject->showEnvironmentDebugMessage($debugMessage,"heizungssteuerung_trigger.class[setRaumTemperatur]",9);
	   if (!$raumSteuerungsTechnikDefiniert) {
		   IPS_LogMessage("trigger.class",$debugMessage);
			return -1;
			}

	   // #########################################
	   // ###
	   // ### KEINE erweiterte Raum-Steuerung
	   // ###
		if ($raumSteuerungsTechnik == "KEINE") {
			// gibt es eine neue Temperatur zu setzen?
			if ($actualSollTemperatur != $actualWunschTemperatur) {
				// ==> Wunsch Temperatur einfach setzen
				$this->configObject->showEnvironmentDebugMessage($grund ." (NORMALE Steuerung)",
					"heizungssteuerung_trigger.class[setRaumTemperatur]",6);
	         $this->heizungsKomponenten->setSollTemperatur($actualRaum,$actualWunschTemperatur,$grund . " (NORMALE Steuerung)");
			}
		} // if ($raumSteuerungsTechnik == "KEINE")

	   // #########################################
	   // ###
	   // ### SIMPLE-DELTA Raum-Steuerung
	   // ###
		if ($raumSteuerungsTechnik == "SIMPLE-DELTA") {
			// $istTemperatur = $this->configObject->getRaumIstTemperatur($actualRaum);
			$deltaTemperatur = $this->configObject->getSimpleDeltaTemperaturRaum($actualRaum);
			$this->configObject->showEnvironmentDebugMessage('Wunsch-Temperaturnderung SIMPLE-DELTA',"heizungssteuerung_trigger.class[setRaumTemperatur]",9);
			$this->configObject->showEnvironmentDebugMessage('   Raum: '. $actualRaum,"heizungssteuerung_trigger.class[setRaumTemperatur]",8);
			$this->configObject->showEnvironmentDebugMessage('   Wunsch-Temperatur: '. $actualWunschTemperatur,"heizungssteuerung_trigger.class[setRaumTemperatur]",8);
			$this->configObject->showEnvironmentDebugMessage('   aktuelle Soll-Tempertaur: '. $actualSollTemperatur,"heizungssteuerung_trigger.class[setRaumTemperatur]",8);
			// $this->configObject->showEnvironmentDebugMessage('   aktuelle Ist-Tempertaur: '. $istTemperatur,"heizungssteuerung_trigger.class[setRaumTemperatur]",8);
			$this->configObject->showEnvironmentDebugMessage('   DeltaTemperatur: '. $deltaTemperatur,"heizungssteuerung_trigger.class[setRaumTemperatur]",8);

			$sollDeltaTemperatur = $actualWunschTemperatur - $deltaTemperatur;
			$this->configObject->showEnvironmentDebugMessage('Aktuelle Soll-Delta-Temperatur ('. $actualWunschTemperatur .' - '. $deltaTemperatur .
				') == '. $sollDeltaTemperatur ."","heizungssteuerung_trigger.class[setRaumTemperatur]",7);

			// gibt es eine neue Temperatur zu setzen?
			if ($actualSollTemperatur != $sollDeltaTemperatur) {
				// Soll-Temperatur um den Delta-Wert angepasst setzen
				$this->configObject->showEnvironmentDebugMessage($grund .'\n(SIMPLE-DELTA Steuerung / Wunschtemperatur == '. $actualWunschTemperatur .')'
					,"heizungssteuerung_trigger.class[setRaumTemperatur]",8);
		     	$this->heizungsKomponenten->setSollTemperatur($actualRaum,$sollDeltaTemperatur,$grund .'\n(SIMPLE-DELTA Steuerung / Wunschtemperatur == '. $actualWunschTemperatur .')');
		   }
     		// } // ELSE von ist die aktuelle Temperatur hher als die gewnschte? ==> keine Anwendung von Delta-Temperatur
		} // $raumSteuerungsTechnik == "SIMPLE-DELTA"
		
	   // #########################################
	   // ###
	   // ### SOLL-IST Vergleich Raum-Steuerung
	   // ###
		if ($raumSteuerungsTechnik == "SOLL-IST-VERGLEICH") {
			$istTemperatur = $this->configObject->getRaumIstTemperatur($actualRaum);
			$deltaTemperatur = $this->configObject->getSimpleDeltaTemperaturRaum($actualRaum);

			$objectIDRaumHeizStatus = $this->configObject->getObjectIDRaumHeizStatus($actualRaum);
			// aktuelle RaumHeizStatus auslesen
			$objectRaumHeizStatus = IPS_GetVariable($objectIDRaumHeizStatus);
			$raumHeizStatusVariableValueArray = $objectRaumHeizStatus['VariableValue'];
			// Boolean ist entweder leer oder enthllt 1 als Inhalt
			// print_r($actualVariableObject);
			$raumHeizStatus = $raumHeizStatusVariableValueArray['ValueBoolean'];
			$raumHeizStatusProfilString = $objectRaumHeizStatus['VariableProfile'];
			if ($raumHeizStatusProfilString == "") {
			   // echo "kein normales Variablen-Profil ausgewhlt\n";
			   // Falls kein eigenes Profil angegeben wurde
				$raumHeizStatusProfilString = $objectRaumHeizStatus['VariableCustomProfile'];
			}
			$raumHeizStatusProfilStringArray = IPS_GetVariableProfile($raumHeizStatusProfilString);
         $raumHeizStatusProfilStringFalse = $raumHeizStatusProfilStringArray['Associations'][0]['Name'];
			// $this->configObject->showEnvironmentDebugMessage('   $raumHeizStatusProfilStringFalse: '. $raumHeizStatusProfilStringFalse,"heizungssteuerung_trigger.class[SOLL-IST-VERGLEICH]",7);
			// print_r($raumHeizStatusProfilStringFalse);
			$raumHeizStatusProfilStringTrue = $raumHeizStatusProfilStringArray['Associations'][1]['Name'];
			// $this->configObject->showEnvironmentDebugMessage('   $raumHeizStatusProfilStringTrue: '. $raumHeizStatusProfilStringTrue,"heizungssteuerung_trigger.class[SOLL-IST-VERGLEICH]",7);

			if ($raumHeizStatus == "") {$raumHeizStatus = 0;}
			$raumHeizStatusString = "";
			if ($raumHeizStatus == 0) {$raumHeizStatusString = $raumHeizStatusProfilStringFalse;}
			if ($raumHeizStatus == 1) {$raumHeizStatusString = $raumHeizStatusProfilStringTrue;}

			$this->configObject->showEnvironmentDebugMessage('Wunsch-Temperaturnderung SOLL-IST-VERGLEICH',"heizungssteuerung_trigger.class[SOLL-IST-VERGLEICH]",9);
			$this->configObject->showEnvironmentDebugMessage('   Raum: '. $actualRaum,"heizungssteuerung_trigger.class[SOLL-IST-VERGLEICH]",8);
			$this->configObject->showEnvironmentDebugMessage('   Wunsch-Temperatur: '. $actualWunschTemperatur,"heizungssteuerung_trigger.class[SOLL-IST-VERGLEICH]",8);
			$this->configObject->showEnvironmentDebugMessage('   aktuelle Soll-Tempertaur: '. $actualSollTemperatur,"heizungssteuerung_trigger.class[SOLL-IST-VERGLEICH]",8);
			$this->configObject->showEnvironmentDebugMessage('   aktuelle Ist-Tempertaur: '. $istTemperatur,"heizungssteuerung_trigger.class[SOLL-IST-VERGLEICH]",8);
			$this->configObject->showEnvironmentDebugMessage('   DeltaTemperatur: '. $deltaTemperatur,"heizungssteuerung_trigger.class[SOLL-IST-VERGLEICH]",8);
			$this->configObject->showEnvironmentDebugMessage('   objectIDRaumHeizStatus: '. $objectIDRaumHeizStatus,"heizungssteuerung_trigger.class[SOLL-IST-VERGLEICH]",8);
			$this->configObject->showEnvironmentDebugMessage('   raumHeizStatus: '. $raumHeizStatus . " (". $raumHeizStatusString .")","heizungssteuerung_trigger.class[SOLL-IST-VERGLEICH]",8);

			$sollDeltaMinusTemperatur = $actualWunschTemperatur - $deltaTemperatur;
			$this->configObject->showEnvironmentDebugMessage('Aktuelle Soll-Delta-Minus-Temperatur ('. $actualWunschTemperatur .' - '. $deltaTemperatur .
				') == '. $sollDeltaMinusTemperatur ."","heizungssteuerung_trigger.class[SOLL-IST-VERGLEICH]",7);
			$sollDeltaPlusTemperatur = $actualWunschTemperatur + $deltaTemperatur;
			$this->configObject->showEnvironmentDebugMessage('Aktuelle Soll-Delta-Plus-Temperatur ('. $actualWunschTemperatur .' + '. $deltaTemperatur .
				') == '. $sollDeltaPlusTemperatur ."","heizungssteuerung_trigger.class[SOLL-IST-VERGLEICH]",7);

			// Fall 1: aktuelle IST-Temperatur ist unterhalb der sollDeltaMinusTemperatur
			if ($istTemperatur < $sollDeltaMinusTemperatur) {
			   // ab jetzt HEIZEN
				// gibt es eine neue Temperatur zu setzen?
				if ($actualSollTemperatur != $sollDeltaPlusTemperatur) {
				   $lokalerGrund = ' - Neuer Raum-Heiz-Status HEIZEN \n(SOLL-IST-VERGLEICH Steuerung / WT == '. $actualWunschTemperatur .''
						. ' IT == '. $istTemperatur .' ==> ST = '. $sollDeltaMinusTemperatur . ")";
					// ==> Plus-Temperatur setzen
					$this->configObject->showEnvironmentDebugMessage($grund . $lokalerGrund,"heizungssteuerung_trigger.class[SOLL-IST-VERGLEICH]",8);
			     	$this->heizungsKomponenten->setHeizenJaNein($actualRaum,TRUE,$sollDeltaPlusTemperatur,$grund . $lokalerGrund);
				}
			}

			// Fall 2: aktuelle IST-Temperatur ist zwischen sollDeltaMinusTemperatur und sollDeltaPlusTemperatur
			if ($istTemperatur >= $sollDeltaMinusTemperatur && $istTemperatur <= $sollDeltaPlusTemperatur) {
			   // ==> der Raum "heizt" entweder oder "khlt ab"
            $sollDeltaTemperatur = -1;
            if ($raumHeizStatus == 0) {$raumHeizStatus = FALSE;}
            if ($raumHeizStatus) {$sollDeltaTemperatur = $sollDeltaPlusTemperatur;}
            else {$sollDeltaTemperatur = $sollDeltaMinusTemperatur;}
            
			   // schauen, ob die aktuelle SOLL-Temperatur aktuell schon stimmt
			   // (im Falle, dass die Wunsch-Temperatur angepasst wurde,
				// kann es passieren, dass Wunsch- und Ist-Temperatur bereinstimmen,
				// aber die SOLL-Temperatur nicht korrekt gesetzt ist
				if ($actualSollTemperatur != $sollDeltaTemperatur) {
				   $lokalerGrund = ' - Neue SOLL-Temperatur, keine Raum-Stati nderung \n(SOLL-IST-VERGLEICH Steuerung / WT == '. $actualWunschTemperatur .''
						. ' IT == '. $istTemperatur .' ==> ST = '. $sollDeltaMinusTemperatur . ")";
					// ==> aktuelle Temperatur setzen
					$this->configObject->showEnvironmentDebugMessage($grund . $lokalerGrund,"heizungssteuerung_trigger.class[SOLL-IST-VERGLEICH]",8);
			     	$this->heizungsKomponenten->setHeizenJaNein($actualRaum,$raumHeizStatus,$sollDeltaTemperatur,$grund . $lokalerGrund);
				}
			}

			// Fall 3: aktuelle IST-Temperatur ist grer als sollDeltaPlusTemperatur
			if ($istTemperatur > $sollDeltaPlusTemperatur) {
				// Minus-Temperatur setzen
				// gibt es eine neue Temperatur zu setzen?
				if ($actualSollTemperatur != $sollDeltaMinusTemperatur) {
					// ==> Plus-Temperatur setzen
					$lokalerGrund = ' - Neuer Raum-Heiz-Status ABKHLEN \n(SOLL-IST-VERGLEICH Steuerung / WT == '.
						$actualWunschTemperatur .' IT == '. $istTemperatur .' ==> ST = '. $sollDeltaMinusTemperatur . ")";
					$this->configObject->showEnvironmentDebugMessage($grund . $lokalerGrund,"heizungssteuerung_trigger.class[SOLL-IST-VERGLEICH]",8);
			     	$this->heizungsKomponenten->setHeizenJaNein($actualRaum,FALSE,$sollDeltaMinusTemperatur,$grund . $lokalerGrund);
				}
			}
		} // $raumSteuerungsTechnik == "SOLL-IST-VERGLEICH"


		// #########################################################
		// ###
		// ### UEBERSCHWAPPT-DELTA Raum-Steuerung
		// ###
		/*
		$ueberschwappTemperaturGrenze = $actualWunschTemperatur - ($deltaTemperatur * $ueberschwapptDelta);
		$this->configObject->showEnvironmentDebugMessage('Aktuelle berschwappt-Temperatur-Grenze ('. $actualWunschTemperatur .' - ('. $deltaTemperatur .
			'* '. $ueberschwapptDelta .')) == '. $ueberschwappTemperaturGrenze ."","heizungssteuerung_trigger.class[advancedHomeMaticV2]",8);

		// #########################################
		// ###
		// ### Aktuelle SOLL-Temperatur setzen:
		// ###
		// FEATURE: eventuell den ganzen Raum immer an der berschwappt-Grenze halten?
		// ist die aktuelle Temperatur hher als die gewnschte? ==> keine Anwendung von Delta-Temperatur

		if ($istTemperatur > $actualWunschTemperatur) {
		  // TODO: kann das berhaupt eintreten bzw. ist das wichtig?
			$this->configObject->showEnvironmentDebugMessage('eventuelle Temperaturabsenkung '. $actualRaum .' auf '. $actualWunschTemperatur .' (falls Soll-Temperatur != Wunsch-Temperatur)',
				"heizungssteuerung_trigger.class[advancedHomeMaticV2]",8);
			// gibt es eine neue Temperatur zu setzen?
			if ($actualSollTemperatur != $actualWunschTemperatur) {
			   // Raum ist wrmer als
				$this->configObject->showEnvironmentDebugMessage('Temperaturabsenkung '. $actualRaum .' auf '. $actualWunschTemperatur .' durch An/Abwesenheit im Zeitprofil ausgelst'
					,"heizungssteuerung_trigger.class[advancedHomeMaticV2]",9);
        		$this->heizungsKomponenten->setSollTemperatur($actualRaum,$actualWunschTemperatur,"Temperatur-Absenkung durch An/Abwesenheit im Zeitprofil ausgelst (Erweiterte HomeMatic Firmware v2.0 Heizungssteuerung)");
        		}
			}
		// else {
		*/

		/*
	   if ($istTemperatur < $ueberschwappTemperaturGrenze) {
	      // Raum sollte "hochgeheizt" werden
			if ($actualSollTemperatur != $actualWunschTemperatur) {
			   // hier: != sollDeltaTemperatur ??
				// gibt es eine neue Temperatur zu setzen?
				$this->configObject->showEnvironmentDebugMessage('Temperaturanhebung '. $actualRaum .' auf '. $sollDeltaTemperatur .' (Delta-Temperatur)'
					,"heizungssteuerung_trigger.class[advancedHomeMaticV2]",9);
           		$this->heizungsKomponenten->setSollTemperatur($actualRaum,$sollDeltaTemperatur,"Delta-Temperatur-Anhebung (Ist-Temperatur == ". $istTemperatur .") ausgelst (Erweiterte HomeMatic Firmware v2.0 Heizungssteuerung)");
           	}
			}
		else {
		   // Raum ist bereits "hochgeheizt" und soll nun "gehalten werden"
			if ($actualSollTemperatur != $actualWunschTemperatur) {
				// gibt es eine neue Temperatur zu setzen?
				$this->configObject->showEnvironmentDebugMessage('Temperaturanhebung '. $actualRaum .' auf '. $actualWunschTemperatur .' (Wunsch-Temperatur)'
					,"heizungssteuerung_trigger.class[advancedHomeMaticV2]",9);
		 		$this->heizungsKomponenten->setSollTemperatur($actualRaum,$actualWunschTemperatur,"Wunsch-Temperatur-Anhebung (Ist-Temperatur == ". $istTemperatur .") ausgelst (Erweiterte HomeMatic Firmware v2.0 Heizungssteuerung)");
			}
		}
		*/

	$this->configObject->showEnvironmentDebugMessage("","heizungssteuerung_trigger.class[setRaumTemperatur]",9);

	} // ende von public function setRaumTemperatur(

} // ende von class HeizungsTrigger

// zum Testen des Script direkt im Editor (Auszug aus heizungssteuerung_aktionsScript-trigger.ips.php):

/*
if (!defined("INCLUDED_TRIGGER")) {
	DEFINE("INCLUDED_TRIGGER",TRUE);
	}
include("includes.ips.php");
$heizungsTriggerObject = new HeizungsTrigger($configObject);
$heizungsTriggerObject->initialize();
*/

?>